@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "core.h"
@@ -319,7 +319,7 @@ static char *expando_last_invite(SERVER_REC *server, void *item, int *free_ret)
 /* client version text string */
 static char *expando_version(SERVER_REC *server, void *item, int *free_ret)
 {
-	return IRSSI_VERSION;
+	return PACKAGE_VERSION;
 }
 
 /* current value of CMDCHARS */
