@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -31,6 +31,7 @@
 #include "channels.h"
 #include "nicklist.h"
 #include "ignore.h"
+#include "recode.h"
 
 #include "window-items.h"
 #include "fe-queries.h"
@@ -38,6 +39,10 @@
 #include "printtext.h"
 
 #define ishighalnum(c) ((unsigned char) (c) >= 128 || i_isalnum(c))
+#define isnickchar(a) \
+	(i_isalnum(a) || (a) == '`' || (a) == '-' || (a) == '_' || \
+	(a) == '[' || (a) == ']' || (a) == '{' || (a) == '}' || \
+	(a) == '|' || (a) == '\\' || (a) == '^')
 
 GHashTable *printnicks;
 
@@ -80,12 +85,25 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 			   use emphasis on them. */
 			int found;
                         char c;
+			char *end2; 
 
+			/* check if _foo_ is a nick */
 			c = end[1];
                         end[1] = '\0';
                         found = nicklist_find(CHANNEL(item), bgn) != NULL;
 			end[1] = c;
 			if (found) continue;
+			
+			/* check if the whole 'word' (e.g. "_foo_^") is a nick
+			   in "_foo_^ ", end will be the second _, end2 the ^ */
+			end2 = end;
+			while (isnickchar(end2[1]))
+				end2++;
+			c = end2[1];
+			end2[1] = '\0';
+			found = nicklist_find(CHANNEL(item), bgn) != NULL;
+			end2[1] = c;
+			if (found) continue;
 		}
 
 		/* allow only *word* emphasis, not *multiple words* */
@@ -116,17 +134,25 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 static char *channel_get_nickmode_rec(NICK_REC *nickrec)
 {
         char *emptystr;
+	char *nickmode;
 
 	if (!settings_get_bool("show_nickmode"))
-                return "";
+                return g_strdup("");
 
         emptystr = settings_get_bool("show_nickmode_empty") ? " " : "";
 
-	return nickrec == NULL ? emptystr :
-		nickrec->op ? "@" :
-		nickrec->halfop ? "%" :
-		nickrec->voice ? "+" :
-		emptystr;
+	if (nickrec == NULL)
+		nickmode = g_strdup(emptystr);
+	else if (nickrec->other) {
+		nickmode = g_malloc(2);
+		nickmode[0] = nickrec->other;
+		nickmode[1] = '\0';
+	} else
+		nickmode = g_strdup(nickrec->op ? "@" :
+				    nickrec->halfop ? "%" :
+				    nickrec->voice ? "+" :
+				    emptystr);
+	return nickmode;
 }
 
 char *channel_get_nickmode(CHANNEL_REC *channel, const char *nick)
@@ -142,9 +168,10 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 			       const char *target, NICK_REC *nickrec)
 {
 	CHANNEL_REC *chanrec;
-	const char *nickmode, *printnick;
+	const char *printnick;
 	int for_me, print_channel, level;
-	char *color, *freemsg = NULL;
+	char *nickmode, *color, *freemsg = NULL;
+	HILIGHT_REC *hilight;
 
 	/* NOTE: this may return NULL if some channel is just closed with
 	   /WINDOW CLOSE and server still sends the few last messages */
@@ -154,8 +181,9 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 
 	for_me = !settings_get_bool("hilight_nick_matches") ? FALSE :
 		nick_match_msg(chanrec, msg, server->nick);
-	color = for_me ? NULL :
+	hilight = for_me ? NULL :
 		hilight_match_nick(server, target, nick, address, MSGLEVEL_PUBLIC, msg);
+	color = (hilight == NULL) ? NULL : hilight_get_color(hilight);
 
 	print_channel = chanrec == NULL ||
 		!window_item_is_active((WI_ITEM_REC *) chanrec);
@@ -164,7 +192,7 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 		print_channel = TRUE;
 
 	level = MSGLEVEL_PUBLIC;
-	if (for_me || color != NULL)
+	if (for_me)
 		level |= MSGLEVEL_HILIGHT;
 
 	if (settings_get_bool("emphasis"))
@@ -178,34 +206,32 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 	if (printnick == NULL)
 		printnick = nick;
 
-	if (!print_channel) {
-		/* message to active channel in window */
-		if (color != NULL) {
-			/* highlighted nick */
-			printformat(server, target, level,
-				    TXT_PUBMSG_HILIGHT,
-				    color, printnick, msg, nickmode);
-		} else {
+	if (color != NULL) {
+		/* highlighted nick */
+		TEXT_DEST_REC dest;
+		format_create_dest(&dest, server, target, level, NULL);
+		hilight_update_text_dest(&dest,hilight);
+		if (!print_channel) /* message to active channel in window */
+			printformat_dest(&dest, TXT_PUBMSG_HILIGHT, color,
+				         printnick, msg, nickmode);
+		else /* message to not existing/active channel */
+			printformat_dest(&dest, TXT_PUBMSG_HILIGHT_CHANNEL,
+					 color, printnick, target, msg,
+					 nickmode);
+	} else {
+		if (!print_channel)
 			printformat(server, target, level,
 				    for_me ? TXT_PUBMSG_ME : TXT_PUBMSG,
 				    printnick, msg, nickmode);
-		}
-	} else {
-		/* message to not existing/active channel */
-		if (color != NULL) {
-			/* highlighted nick */
-			printformat(server, target, level,
-				    TXT_PUBMSG_HILIGHT_CHANNEL,
-				    color, printnick, target, msg, nickmode);
-		} else {
+		else
 			printformat(server, target, level,
 				    for_me ? TXT_PUBMSG_ME_CHANNEL :
 				    TXT_PUBMSG_CHANNEL,
 				    printnick, target, msg, nickmode);
-		}
-	}
+	}						
 
-        g_free_not_null(freemsg);
+	g_free_not_null(nickmode);
+	g_free_not_null(freemsg);
 	g_free_not_null(color);
 }
 
@@ -233,9 +259,8 @@ static void sig_message_own_public(SERVER_REC *server, const char *msg,
 	WINDOW_REC *window;
 	CHANNEL_REC *channel;
 	const char *nickmode;
-        char *freemsg = NULL;
+        char *freemsg = NULL, *recoded;
 	int print_channel;
-
 	channel = channel_find(server, target);
 	if (channel != NULL)
 		target = channel->visible_name;
@@ -255,14 +280,18 @@ static void sig_message_own_public(SERVER_REC *server, const char *msg,
 	if (settings_get_bool("emphasis"))
 		msg = freemsg = expand_emphasis((WI_ITEM_REC *) channel, msg);
 
+	/* ugly: recode the sent message back for printing */ 
+	recoded = recode_in(server, msg, target);
+	
 	if (!print_channel) {
 		printformat(server, target, MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
-			    TXT_OWN_MSG, server->nick, msg, nickmode);
+			    TXT_OWN_MSG, server->nick, recoded, nickmode);
 	} else {
 		printformat(server, target, MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
-			    TXT_OWN_MSG_CHANNEL, server->nick, target, msg, nickmode);
+			    TXT_OWN_MSG_CHANNEL, server->nick, target, recoded, nickmode);
 	}
 
+	g_free(recoded);
 	g_free_not_null(freemsg);
 }
 
@@ -270,11 +299,10 @@ static void sig_message_own_private(SERVER_REC *server, const char *msg,
 				    const char *target, const char *origtarget)
 {
 	QUERY_REC *query;
-        char *freemsg = NULL;
+        char *freemsg = NULL, *recoded;
 
 	g_return_if_fail(server != NULL);
 	g_return_if_fail(msg != NULL);
-
 	if (target == NULL) {
 		/* this should only happen if some special target failed and
 		   we should display some error message. currently the special
@@ -294,11 +322,15 @@ static void sig_message_own_private(SERVER_REC *server, const char *msg,
 	if (settings_get_bool("emphasis"))
 		msg = freemsg = expand_emphasis((WI_ITEM_REC *) query, msg);
 
+	/* ugly: recode the sent message back for printing */
+	recoded = recode_in(server, msg, target);
+
 	printformat(server, target,
 		    MSGLEVEL_MSGS | MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
 		    query == NULL ? TXT_OWN_MSG_PRIVATE :
-		    TXT_OWN_MSG_PRIVATE_QUERY, target, msg, server->nick);
+		    TXT_OWN_MSG_PRIVATE_QUERY, target, recoded, server->nick);
 
+	g_free(recoded);
 	g_free_not_null(freemsg);
 }
 
@@ -379,6 +411,7 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 	if (once || count == 0) {
 		if (chans->len > 0)
 			g_string_truncate(chans, chans->len-1);
+		/* at least recode_fallback will be used */
 		printformat(server, print_channel, MSGLEVEL_QUITS,
 			    count <= 1 ? TXT_QUIT : TXT_QUIT_ONCE,
 			    nick, address, reason, chans->str);
@@ -422,7 +455,7 @@ static void print_nick_change(SERVER_REC *server, const char *newnick,
 
 	msgprint = FALSE;
 
-	/* Print to each channel/query where the nick is.
+	/* Print to each channel where the nick is.
 	   Don't print more than once to the same window. */
 	windows = NULL;
 	for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
