@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -124,8 +124,8 @@ static void hilight_init_rec(HILIGHT_REC *rec)
 void hilight_create(HILIGHT_REC *rec)
 {
 	if (g_slist_find(hilights, rec) != NULL) {
-		hilights = g_slist_remove(hilights, rec);
 		hilight_remove_config(rec);
+		hilights = g_slist_remove(hilights, rec);
 	}
 
 	hilights = g_slist_append(hilights, rec);
@@ -279,7 +279,7 @@ static char *hilight_get_act_color(HILIGHT_REC *rec)
 			settings_get_str("hilight_act_color"));
 }
 
-static char *hilight_get_color(HILIGHT_REC *rec)
+char *hilight_get_color(HILIGHT_REC *rec)
 {
 	const char *color;
 
@@ -291,7 +291,7 @@ static char *hilight_get_color(HILIGHT_REC *rec)
 	return format_string_expand(color, NULL);
 }
 
-static void hilight_update_text_dest(TEXT_DEST_REC *dest, HILIGHT_REC *rec)
+void hilight_update_text_dest(TEXT_DEST_REC *dest, HILIGHT_REC *rec)
 {
 	dest->level |= MSGLEVEL_HILIGHT;
 
@@ -305,6 +305,8 @@ static void hilight_update_text_dest(TEXT_DEST_REC *dest, HILIGHT_REC *rec)
 		dest->hilight_color = hilight_get_act_color(rec);
 }
 
+static void hilight_print(int index, HILIGHT_REC *rec);
+
 static void sig_print_text(TEXT_DEST_REC *dest, const char *text,
 			   const char *stripped)
 {
@@ -398,19 +400,15 @@ static void sig_print_text(TEXT_DEST_REC *dest, const char *text,
 	signal_stop();
 }
 
-char *hilight_match_nick(SERVER_REC *server, const char *channel,
+HILIGHT_REC *hilight_match_nick(SERVER_REC *server, const char *channel,
 			 const char *nick, const char *address,
 			 int level, const char *msg)
 {
         HILIGHT_REC *rec;
-	char *color;
 
 	rec = hilight_match(server, channel, nick, address,
 			    level, msg, NULL, NULL);
-	color = rec == NULL || !rec->nick ? NULL :
-		hilight_get_color(rec);
-
-	return color;
+	return (rec == NULL || !rec->nick) ? NULL : rec;
 }
 
 static void read_hilight_config(void)
@@ -502,6 +500,8 @@ static void hilight_print(int index, HILIGHT_REC *rec)
 		g_strjoinv(",", rec->channels);
 	levelstr = rec->level == 0 ? NULL :
 		bits2level(rec->level);
+	if (levelstr != NULL)
+		levelstr = g_strconcat(levelstr, " ", NULL);
 	printformat(NULL, NULL, MSGLEVEL_CLIENTCRAP,
 		    TXT_HILIGHT_LINE, index, rec->text,
 		    chans != NULL ? chans : "",
