@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -220,6 +220,7 @@ FORMAT_REC fecommon_core_formats[] = {
 	{ "invalid_size", "Invalid size", 0 },
 	{ "invalid_charset", "Invalid charset: $0", 1, { 0 } },
 	{ "eval_max_recurse", "/eval hit maximum recursion limit", 0 },
+	{ "program_not_found", "Could not find file or file is not executable", 0 },
 
 	/* ---- */
 	{ NULL, "Themes", 0 },
