@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -1249,12 +1249,13 @@ static void sig_complete_format(GList **list, WINDOW_REC *window,
         ptr = line;
 
 	words = 0;
-	do {
-                ptr++;
-
-		words++;
-                ptr = strchr(ptr, ' ');
-	} while (ptr != NULL);
+	if (*ptr != '\0') {
+		do {
+			ptr++;
+			words++;
+			ptr = strchr(ptr, ' ');
+		} while (ptr != NULL);
+	}
 
 	if (words > 2)
 		return;
