@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -115,9 +115,9 @@ static void event_who(IRC_SERVER_REC *server, const char *data)
 	/* split hops/realname */
 	hops = realname;
 	while (*realname != '\0' && *realname != ' ') realname++;
-	while (*realname == ' ') realname++;
-	if (realname > hops) realname[-1] = '\0';
-
+	if (*realname == ' ')
+		*realname++ = '\0';
+	
 	printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_WHO,
 		    channel, nick, stat, hops, user, host, realname, serv);
 
@@ -195,6 +195,17 @@ static void event_silence_list(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
+static void event_accept_list(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *accepted;
+
+	g_return_if_fail(data != NULL);
+	g_return_if_fail(server != NULL);
+
+	params = event_get_params(data, 2, NULL, &accepted);
+	printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_ACCEPT_LIST, accepted);
+	g_free(params);
+}
 
 static void event_invite_list(IRC_SERVER_REC *server, const char *data)
 {
@@ -350,289 +361,6 @@ static void event_sent_invite(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
-static void event_whois(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *user, *host, *realname;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 6, NULL, &nick, &user,
-				  &host, NULL, &realname);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS, nick, user, host, realname);
-	g_free(params);
-}
-
-static void event_whois_idle(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *secstr, *signonstr, *rest, *timestr;
-	long days, hours, mins, secs;
-	time_t signon;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 5 | PARAM_FLAG_GETREST, NULL,
-				  &nick, &secstr, &signonstr, &rest);
-
-	secs = atol(secstr);
-	signon = strstr(rest, "signon time") == NULL ? 0 :
-		(time_t) atol(signonstr);
-
-	days = secs/3600/24;
-	hours = (secs%(3600*24))/3600;
-	mins = (secs%3600)/60;
-	secs %= 60;
-
-	if (signon == 0)
-		printformat(server, nick, MSGLEVEL_CRAP, IRCTXT_WHOIS_IDLE,
-			    nick, days, hours, mins, secs);
-	else {
-		timestr = my_asctime(signon);
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_WHOIS_IDLE_SIGNON,
-			    nick, days, hours, mins, secs, timestr);
-		g_free(timestr);
-	}
-	g_free(params);
-}
-
-static void event_whois_server(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *whoserver, *desc;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 4, NULL, &nick, &whoserver, &desc);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_SERVER, nick, whoserver, desc);
-	g_free(params);
-}
-
-static void event_whois_oper(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *type;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3, NULL, &nick, &type);
-	/* type = "is an IRC Operator" */
-	if (strlen(type) > 5) {
-		type += 5;
-		if (*type == ' ') type++;
-	}
-	if (*type == '\0') {
-		/* shouldn't happen */
-		type = "IRC Operator";
-	}
-
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_OPER, nick, type);
-	g_free(params);
-}
-
-static void event_whois_registered(IRC_SERVER_REC *server, const char *data,
-				   const char *orignick, const char *addr)
-{
-	char *params, *nick, *txt_identified;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3, NULL, &nick, &txt_identified);
-	if (*txt_identified != '\0') {
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_WHOIS_REGISTERED, nick);
-	} else {
-		/* or /USERIP reply in undernet.. */
-		print_event_received(server, data, orignick, FALSE);
-	}
-	g_free(params);
-}
-
-static void event_whois_help(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 2, NULL, &nick);
-	printformat(server, nick, MSGLEVEL_CRAP, IRCTXT_WHOIS_HELP, nick);
-	g_free(params);
-}
-
-static void event_whois_modes(IRC_SERVER_REC *server, const char *data,
-			      const char *orignick)
-{
-	char *params, *nick, *modes;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 6, NULL, &nick,
-				  NULL, NULL, NULL, &modes);
-
-	if (!ischannel(*nick)) {
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_WHOIS_MODES, nick, modes);
-	} else {
-		/* OPN's dancer uses for channel forwarding */
-		print_event_received(server, data, orignick, FALSE);
-	}
-	g_free(params);
-}
-
-static void event_whois_realhost(IRC_SERVER_REC *server, const char *data,
-				 const char *orignick, const char *addr)
-{
-	char *params, *nick, *txt_real, *txt_hostname, *hostname;
-
-	g_return_if_fail(data != NULL);
-
-        /* <yournick> real hostname <nick> <hostname> */
-	params = event_get_params(data, 5, NULL, &nick, &txt_real,
-				  &txt_hostname, &hostname);
-	if (strcmp(txt_real, "real") != 0 ||
-	    strcmp(txt_hostname, "hostname") != 0) {
-		/* <yournick> <nick> :... from <hostname> */
-                g_free(params);
-		params = event_get_params(data, 3, NULL, &nick, &hostname);
-
-		hostname = strstr(hostname, "from ");
-                if (hostname != NULL) hostname += 5;
-	}
-
-	if (hostname != NULL) {
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_WHOIS_REALHOST, nick, hostname, "");
-	} else {
-		/* OPN's dancer uses for end of /MAP */
-		print_event_received(server, data, orignick, FALSE);
-	}
-	g_free(params);
-}
-
-static void event_whois_usermode326(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *usermode;
-
-	g_return_if_fail(data != NULL);
-
-        /* <yournick> <nick> :has oper privs: <mode> */
-	params = event_get_params(data, 3, NULL, &nick, &usermode);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_USERMODE, nick, usermode);
-        g_free(params);
-}
-
-static void event_whois_realhost327(IRC_SERVER_REC *server, const char *data,
-				    const char *orignick, const char *addr)
-{
-	char *params, *nick, *hostname, *ip, *text;
-
-	g_return_if_fail(data != NULL);
-
-	/* <yournick> <hostname> <ip> :Real hostname/IP */
-	params = event_get_params(data, 5, NULL, &nick, &hostname, &ip, &text);
-	if (*text != '\0') {
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_WHOIS_REALHOST, nick, hostname, ip);
-	} else {
-		print_event_received(server, data, orignick, FALSE);
-	}
-	g_free(params);
-}
-
-static void event_whois_usermode(IRC_SERVER_REC *server, const char *data,
-				 const char *orignick, const char *addr)
-{
-	char *params, *txt_usermodes, *nick, *usermode;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 4, NULL, &txt_usermodes,
-				  &nick, &usermode);
-
-	if (strcmp(txt_usermodes, "usermodes") == 0) {
-		/* <yournick> usermodes <nick> usermode */
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_WHOIS_USERMODE, nick, usermode);
-	} else {
-		/* some servers use this as motd too..
-		   and OPN's dancer for /MAP */
-		print_event_received(server, data, orignick, FALSE);
-	}
-	g_free(params);
-}
-
-static void event_whois_special(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *str;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3, NULL, &nick, &str);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_SPECIAL, nick, str);
-	g_free(params);
-}
-
-static void event_whowas(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *user, *host, *realname;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 6, NULL, &nick, &user,
-				  &host, NULL, &realname);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOWAS, nick, user, host, realname);
-	g_free(params);
-}
-
-static void event_whois_channels(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *chans;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3, NULL, &nick, &chans);
-
-	/* sure - we COULD print the channel names as-is, but since the
-	   colors, bolds, etc. are mostly just to fool people, I think we
-	   should show the channel names as they REALLY are so they could
-	   even be joined without any extra tricks. */
-        chans = show_lowascii(chans);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_CHANNELS, nick, chans);
-	g_free(chans);
-
-	g_free(params);
-}
-
-static void event_whois_away(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *awaymsg;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3, NULL, &nick, &awaymsg);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_AWAY, nick, awaymsg);
-	g_free(params);
-}
-
-static void event_end_of_whois(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 2, NULL, &nick);
-	if (server->whois_found) {
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_END_OF_WHOIS, nick);
-	}
-	g_free(params);
-}
-
 static void event_chanserv_url(IRC_SERVER_REC *server, const char *data)
 {
 	const char *channel;
@@ -647,32 +375,6 @@ static void event_chanserv_url(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
-static void event_whois_auth(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick, *text;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3, NULL, &nick, &text);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_EXTRA, nick, text);
-	g_free(params);
-}
-
-static void event_end_of_whowas(IRC_SERVER_REC *server, const char *data)
-{
-	char *params, *nick;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 2, NULL, &nick);
-	if (server->whowas_found) {
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    IRCTXT_END_OF_WHOWAS, nick);
-	}
-	g_free(params);
-}
-
 static void event_target_unavailable(IRC_SERVER_REC *server, const char *data,
 				     const char *nick, const char *addr)
 {
@@ -701,14 +403,20 @@ static void event_target_unavailable(IRC_SERVER_REC *server, const char *data,
 	g_free(params);
 }
 
-static void event_no_such_nick(IRC_SERVER_REC *server, const char *data)
+static void event_no_such_nick(IRC_SERVER_REC *server, const char *data,
+				     const char *nick, const char *addr)
 {
-	char *params, *nick;
+	char *params, *unick;
 
 	g_return_if_fail(data != NULL);
 
-	params = event_get_params(data, 2, NULL, &nick);
-	printformat(server, nick, MSGLEVEL_CRAP, IRCTXT_NO_SUCH_NICK, nick);
+	params = event_get_params(data, 2, NULL, &unick);
+	if (!strcmp(unick, "*"))
+		/* more information will be in the description,
+		 * e.g. * :Target left IRC. Failed to deliver: [hi] */
+		print_event_received(server, data, nick, FALSE);
+	else
+		printformat(server, unick, MSGLEVEL_CRAP, IRCTXT_NO_SUCH_NICK, unick);
 	g_free(params);
 }
 
@@ -785,6 +493,26 @@ static void event_bad_channel_mask(IRC_SERVER_REC *server, const char *data)
 	cannot_join(server, data, IRCTXT_JOINERROR_BAD_MASK);
 }
 
+static void event_477(IRC_SERVER_REC *server, const char *data,
+		      const char *nick)
+{
+	/* Numeric 477 can mean many things:
+	 * modeless channel, cannot join/send to channel (+r/+R/+M).
+	 * If we tried to join this channel, display the error in the
+	 * status window. Otherwise display it in the channel window.
+	 */
+	IRC_CHANNEL_REC *chanrec;
+	char *params, *channel;
+
+	g_return_if_fail(data != NULL);
+
+	params = event_get_params(data, 2, NULL, &channel);
+
+	chanrec = irc_channel_find(server, channel);
+	print_event_received(server, data, nick, chanrec == NULL || chanrec->joined);
+	g_free(params);
+}
+
 static void event_unknown_mode(IRC_SERVER_REC *server, const char *data)
 {
 	char *params, *mode;
@@ -807,7 +535,7 @@ static void event_numeric(IRC_SERVER_REC *server, const char *data,
 static void print_event_received(IRC_SERVER_REC *server, const char *data,
 				 const char *nick, int target_param)
 {
-	char *target, *args, *ptr;
+	char *target, *args, *ptr, *ptr2;
 	int format;
 
 	g_return_if_fail(data != NULL);
@@ -818,17 +546,11 @@ static void print_event_received(IRC_SERVER_REC *server, const char *data,
 		return;
 	ptr++;
 
-	if (!target_param)
+	if (!target_param || *ptr == ':' || (ptr2 = strchr(ptr, ' ')) == NULL)
 		target = NULL;
 	else {
-                /* target parameter */
-		target = ptr;
-		ptr = strchr(target, ' ');
-		if (ptr == NULL)
-			return;
-
-                target = g_strndup(target, (int) (ptr-target));
-		ptr++;
+                /* target parameter expected and present */
+                target = g_strndup(ptr, (int) (ptr2-ptr));
 	}
 
 	/* param1 param2 ... :last parameter */
@@ -847,7 +569,9 @@ static void print_event_received(IRC_SERVER_REC *server, const char *data,
 		IRCTXT_DEFAULT_EVENT : IRCTXT_DEFAULT_EVENT_SERVER;
 	printformat(server, target, MSGLEVEL_CRAP, format,
 		    nick, args, current_server_event);
+
 	g_free(args);
+	g_free(target);
 }
 
 static void event_received(IRC_SERVER_REC *server, const char *data,
@@ -890,6 +614,7 @@ void fe_events_numeric_init(void)
 	signal_add("event 315", (SIGNAL_FUNC) event_end_of_who);
 	signal_add("event 271", (SIGNAL_FUNC) event_silence_list);
 	signal_add("event 272", (SIGNAL_FUNC) sig_empty);
+	signal_add("event 281", (SIGNAL_FUNC) event_accept_list);
 	signal_add("event 367", (SIGNAL_FUNC) event_ban_list);
 	signal_add("event 348", (SIGNAL_FUNC) event_eban_list);
 	signal_add("event 346", (SIGNAL_FUNC) event_invite_list);
@@ -901,28 +626,7 @@ void fe_events_numeric_init(void)
 	signal_add("event 306", (SIGNAL_FUNC) event_nowaway);
 	signal_add("event 305", (SIGNAL_FUNC) event_unaway);
 	signal_add("event 301", (SIGNAL_FUNC) event_away);
-	signal_add("event 311", (SIGNAL_FUNC) event_whois);
-	signal_add("whois away", (SIGNAL_FUNC) event_whois_away);
-	signal_add("whowas away", (SIGNAL_FUNC) event_whois_away);
-	signal_add("event 312", (SIGNAL_FUNC) event_whois_server);
-	signal_add("event 313", (SIGNAL_FUNC) event_whois_oper);
-	signal_add("event 307", (SIGNAL_FUNC) event_whois_registered);
-	signal_add("event 310", (SIGNAL_FUNC) event_whois_help);
-	signal_add("event 326", (SIGNAL_FUNC) event_whois_usermode326);
-	signal_add("event 327", (SIGNAL_FUNC) event_whois_realhost327);
-	signal_add("event 379", (SIGNAL_FUNC) event_whois_modes);
-	signal_add("event 378", (SIGNAL_FUNC) event_whois_realhost);
-	signal_add("event 377", (SIGNAL_FUNC) event_whois_usermode);
-	signal_add("event 320", (SIGNAL_FUNC) event_whois_special);
-	signal_add("event 308", (SIGNAL_FUNC) event_whois_special);
-	signal_add("event 275", (SIGNAL_FUNC) event_whois_special);
-	signal_add("event 314", (SIGNAL_FUNC) event_whowas);
-	signal_add("event 317", (SIGNAL_FUNC) event_whois_idle);
-	signal_add("event 318", (SIGNAL_FUNC) event_end_of_whois);
 	signal_add("event 328", (SIGNAL_FUNC) event_chanserv_url);
-	signal_add("event 330", (SIGNAL_FUNC) event_whois_auth);
-	signal_add("event 369", (SIGNAL_FUNC) event_end_of_whowas);
-	signal_add("event 319", (SIGNAL_FUNC) event_whois_channels);
 	signal_add("event 302", (SIGNAL_FUNC) event_userhost);
 	signal_add("event 341", (SIGNAL_FUNC) event_sent_invite);
 
@@ -937,32 +641,54 @@ void fe_events_numeric_init(void)
 	signal_add("event 474", (SIGNAL_FUNC) event_banned);
 	signal_add("event 475", (SIGNAL_FUNC) event_bad_channel_key);
 	signal_add("event 476", (SIGNAL_FUNC) event_bad_channel_mask);
+	signal_add("event 477", (SIGNAL_FUNC) event_477);
 	signal_add("event 375", (SIGNAL_FUNC) event_motd);
 	signal_add("event 376", (SIGNAL_FUNC) event_motd);
 	signal_add("event 372", (SIGNAL_FUNC) event_motd);
 	signal_add("event 422", (SIGNAL_FUNC) event_motd);
 
         signal_add("default event numeric", (SIGNAL_FUNC) event_numeric);
+	/* Because default event numeric only fires if there is no specific
+	 * event, add all numerics with a handler elsewhere in irssi that
+	 * should not be printed specially here.
+	 */
 	signal_add("event 001", (SIGNAL_FUNC) event_received);
 	signal_add("event 004", (SIGNAL_FUNC) event_received);
+	signal_add("event 005", (SIGNAL_FUNC) event_received);
 	signal_add("event 254", (SIGNAL_FUNC) event_received);
 	signal_add("event 364", (SIGNAL_FUNC) event_received);
 	signal_add("event 365", (SIGNAL_FUNC) event_received);
 	signal_add("event 381", (SIGNAL_FUNC) event_received);
+	signal_add("event 396", (SIGNAL_FUNC) event_received);
 	signal_add("event 421", (SIGNAL_FUNC) event_received);
 	signal_add("event 432", (SIGNAL_FUNC) event_received);
 	signal_add("event 436", (SIGNAL_FUNC) event_received);
 	signal_add("event 438", (SIGNAL_FUNC) event_received);
 	signal_add("event 465", (SIGNAL_FUNC) event_received);
 	signal_add("event 439", (SIGNAL_FUNC) event_received);
+	signal_add("event 470", (SIGNAL_FUNC) event_received);
 	signal_add("event 479", (SIGNAL_FUNC) event_received);
-	signal_add("event 482", (SIGNAL_FUNC) event_received);
 
-	signal_add("event 368", (SIGNAL_FUNC) event_target_received);
-	signal_add("event 347", (SIGNAL_FUNC) event_target_received);
-	signal_add("event 349", (SIGNAL_FUNC) event_target_received);
-	signal_add("event 442", (SIGNAL_FUNC) event_target_received);
-	signal_add("event 477", (SIGNAL_FUNC) event_target_received);
+	signal_add("event 344", (SIGNAL_FUNC) event_target_received); /* reop list */
+	signal_add("event 345", (SIGNAL_FUNC) event_target_received); /* end of reop list */
+	signal_add("event 347", (SIGNAL_FUNC) event_target_received); /* end of invite exception list */
+	signal_add("event 349", (SIGNAL_FUNC) event_target_received); /* end of ban exception list */
+	signal_add("event 368", (SIGNAL_FUNC) event_target_received); /* end of ban list */
+	signal_add("event 386", (SIGNAL_FUNC) event_target_received); /* owner list; old rsa challenge (harmless) */
+	signal_add("event 387", (SIGNAL_FUNC) event_target_received); /* end of owner list */
+	signal_add("event 388", (SIGNAL_FUNC) event_target_received); /* protect list */
+	signal_add("event 389", (SIGNAL_FUNC) event_target_received); /* end of protect list */
+	signal_add("event 404", (SIGNAL_FUNC) event_target_received); /* cannot send to channel */
+	signal_add("event 408", (SIGNAL_FUNC) event_target_received); /* cannot send (+c) */
+	signal_add("event 442", (SIGNAL_FUNC) event_target_received); /* you're not on that channel */
+	signal_add("event 478", (SIGNAL_FUNC) event_target_received); /* ban list is full */
+	signal_add("event 482", (SIGNAL_FUNC) event_target_received); /* not chanop */
+	signal_add("event 486", (SIGNAL_FUNC) event_target_received); /* cannot /msg (+R) */
+	signal_add("event 489", (SIGNAL_FUNC) event_target_received); /* not chanop or voice */
+	signal_add("event 494", (SIGNAL_FUNC) event_target_received); /* cannot /msg (own +R) */
+	signal_add("event 506", (SIGNAL_FUNC) event_target_received); /* cannot send (+R) */
+	signal_add("event 716", (SIGNAL_FUNC) event_target_received); /* cannot /msg (+g) */
+	signal_add("event 717", (SIGNAL_FUNC) event_target_received); /* +g notified */
 }
 
 void fe_events_numeric_deinit(void)
@@ -978,6 +704,7 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 315", (SIGNAL_FUNC) event_end_of_who);
 	signal_remove("event 271", (SIGNAL_FUNC) event_silence_list);
 	signal_remove("event 272", (SIGNAL_FUNC) sig_empty);
+	signal_remove("event 281", (SIGNAL_FUNC) event_accept_list);
 	signal_remove("event 367", (SIGNAL_FUNC) event_ban_list);
 	signal_remove("event 348", (SIGNAL_FUNC) event_eban_list);
 	signal_remove("event 346", (SIGNAL_FUNC) event_invite_list);
@@ -989,28 +716,7 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 306", (SIGNAL_FUNC) event_nowaway);
 	signal_remove("event 305", (SIGNAL_FUNC) event_unaway);
 	signal_remove("event 301", (SIGNAL_FUNC) event_away);
-	signal_remove("event 311", (SIGNAL_FUNC) event_whois);
-	signal_remove("whois away", (SIGNAL_FUNC) event_whois_away);
-	signal_remove("whowas away", (SIGNAL_FUNC) event_whois_away);
-	signal_remove("event 312", (SIGNAL_FUNC) event_whois_server);
-	signal_remove("event 313", (SIGNAL_FUNC) event_whois_oper);
-	signal_remove("event 307", (SIGNAL_FUNC) event_whois_registered);
-	signal_remove("event 310", (SIGNAL_FUNC) event_whois_help);
-	signal_remove("event 326", (SIGNAL_FUNC) event_whois_usermode326);
-	signal_remove("event 327", (SIGNAL_FUNC) event_whois_realhost327);
-	signal_remove("event 379", (SIGNAL_FUNC) event_whois_modes);
-	signal_remove("event 378", (SIGNAL_FUNC) event_whois_realhost);
-	signal_remove("event 377", (SIGNAL_FUNC) event_whois_usermode);
-	signal_remove("event 320", (SIGNAL_FUNC) event_whois_special);
-	signal_remove("event 275", (SIGNAL_FUNC) event_whois_special);
-	signal_remove("event 308", (SIGNAL_FUNC) event_whois_special);
-	signal_remove("event 314", (SIGNAL_FUNC) event_whowas);
-	signal_remove("event 317", (SIGNAL_FUNC) event_whois_idle);
-	signal_remove("event 318", (SIGNAL_FUNC) event_end_of_whois);
 	signal_remove("event 328", (SIGNAL_FUNC) event_chanserv_url);
-	signal_remove("event 330", (SIGNAL_FUNC) event_whois_auth);
-	signal_remove("event 369", (SIGNAL_FUNC) event_end_of_whowas);
-	signal_remove("event 319", (SIGNAL_FUNC) event_whois_channels);
 	signal_remove("event 302", (SIGNAL_FUNC) event_userhost);
 	signal_remove("event 341", (SIGNAL_FUNC) event_sent_invite);
 
@@ -1025,6 +731,7 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 474", (SIGNAL_FUNC) event_banned);
 	signal_remove("event 475", (SIGNAL_FUNC) event_bad_channel_key);
 	signal_remove("event 476", (SIGNAL_FUNC) event_bad_channel_mask);
+	signal_remove("event 477", (SIGNAL_FUNC) event_477);
 	signal_remove("event 375", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 376", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 372", (SIGNAL_FUNC) event_motd);
@@ -1033,22 +740,39 @@ void fe_events_numeric_deinit(void)
         signal_remove("default event numeric", (SIGNAL_FUNC) event_numeric);
 	signal_remove("event 001", (SIGNAL_FUNC) event_received);
 	signal_remove("event 004", (SIGNAL_FUNC) event_received);
+	signal_remove("event 005", (SIGNAL_FUNC) event_received);
 	signal_remove("event 254", (SIGNAL_FUNC) event_received);
 	signal_remove("event 364", (SIGNAL_FUNC) event_received);
 	signal_remove("event 365", (SIGNAL_FUNC) event_received);
 	signal_remove("event 381", (SIGNAL_FUNC) event_received);
+	signal_remove("event 396", (SIGNAL_FUNC) event_received);
 	signal_remove("event 421", (SIGNAL_FUNC) event_received);
 	signal_remove("event 432", (SIGNAL_FUNC) event_received);
 	signal_remove("event 436", (SIGNAL_FUNC) event_received);
 	signal_remove("event 438", (SIGNAL_FUNC) event_received);
 	signal_remove("event 465", (SIGNAL_FUNC) event_received);
 	signal_remove("event 439", (SIGNAL_FUNC) event_received);
+	signal_remove("event 470", (SIGNAL_FUNC) event_received);
 	signal_remove("event 479", (SIGNAL_FUNC) event_received);
-	signal_remove("event 482", (SIGNAL_FUNC) event_received);
 
-	signal_remove("event 368", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 344", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 345", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 347", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 349", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 368", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 386", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 387", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 388", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 389", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 404", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 408", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 442", (SIGNAL_FUNC) event_target_received);
-	signal_remove("event 477", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 478", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 482", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 486", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 489", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 494", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 506", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 716", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 717", (SIGNAL_FUNC) event_target_received);
 }
