@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -38,15 +38,28 @@
 #include "fe-queries.h"
 #include "window-items.h"
 
-static const char *skip_target(const char *target)
+static const char *skip_target(IRC_SERVER_REC *server, const char *target)
 {
-	if (target != NULL && *target == '@') {
-		/* @#channel, @+#channel - Hybrid6 / Bahamut features */
-		if (target[1] == '+' && ischannel(target[2]))
-			target += 2;
-		else if (ischannel(target[1]))
-			target++;
-	}
+	int i = 0;
+	const char *val, *chars;
+
+	/* Quick check */
+	if (server->prefix[(int)(unsigned char)*target] == 0)
+		return target;
+
+	/* Hack: for bahamut 1.4 which sends neither STATUSMSG nor
+	 * WALLCHOPS in 005, accept @#chan and @+#chan (but not +#chan) */
+	val = g_hash_table_lookup(server->isupport, "STATUSMSG");
+	if (val == NULL && *target != '@')
+		return target;
+	chars = val ? val : "@+";
+	for(i = 0; target[i] != '\0'; i++) {
+		if (strchr(chars, target[i]) == NULL)
+			break;
+	};
+
+	if(ischannel(target[i]))
+		target += i;
 
 	return target;
 }
@@ -58,7 +71,7 @@ static void sig_message_own_public(SERVER_REC *server, const char *msg,
 	char *nickmode, *recoded;
 
 	oldtarget = target;
-	target = skip_target(target);
+	target = skip_target(IRC_SERVER(server), target);
 	if (IS_IRC_SERVER(server) && target != oldtarget) {
 		/* Hybrid 6 / Bahamut feature, send msg to all
 		   ops / ops+voices in channel */
@@ -72,6 +85,7 @@ static void sig_message_own_public(SERVER_REC *server, const char *msg,
 				   MSGLEVEL_NO_ACT,
 				   TXT_OWN_MSG_CHANNEL,
 				   server->nick, oldtarget, recoded, nickmode);
+		g_free(nickmode);
                 g_free(recoded);
                 signal_stop();
 	}
@@ -93,6 +107,7 @@ static void sig_message_irc_op_public(SERVER_REC *server, const char *msg,
 			   MSGLEVEL_PUBLIC,
 			   TXT_PUBMSG_CHANNEL,
 			   nick, optarget, msg, nickmode);
+	g_free(nickmode);
         g_free(optarget);
 }
 
@@ -112,6 +127,7 @@ static void sig_message_own_wall(SERVER_REC *server, const char *msg,
 			   MSGLEVEL_NO_ACT,
 			   TXT_OWN_MSG_CHANNEL,
 			   server->nick, optarget, recoded, nickmode);
+	g_free(nickmode);
         g_free(recoded);
         g_free(optarget);
 }
@@ -120,8 +136,11 @@ static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
                                    const char *target)
 {
 	void *item;
+	const char *oldtarget;
         char *freemsg = NULL, *recoded;
 
+	oldtarget = target;
+	target = skip_target(IRC_SERVER(server), target);
         if (ischannel(*target))
 		item = irc_channel_find(server, target);
 	else
@@ -136,8 +155,8 @@ static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
 	printformat(server, target,
 		    MSGLEVEL_ACTIONS | MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT |
 		    (ischannel(*target) ? MSGLEVEL_PUBLIC : MSGLEVEL_MSGS),
-		    item != NULL ? IRCTXT_OWN_ACTION : IRCTXT_OWN_ACTION_TARGET,
-		    server->nick, recoded, target);
+		    item != NULL && oldtarget == target ? IRCTXT_OWN_ACTION : IRCTXT_OWN_ACTION_TARGET,
+		    server->nick, recoded, oldtarget);
         g_free(recoded);
         g_free_not_null(freemsg);
 }
@@ -147,9 +166,13 @@ static void sig_message_irc_action(IRC_SERVER_REC *server, const char *msg,
 				   const char *target)
 {
 	void *item;
-        char *freemsg = NULL, *recoded;
+	const char *oldtarget;
+        char *freemsg = NULL;
 	int level;
 
+	oldtarget = target;
+	target = skip_target(IRC_SERVER(server), target);
+
 	level = MSGLEVEL_ACTIONS |
 		(ischannel(*target) ? MSGLEVEL_PUBLIC : MSGLEVEL_MSGS);
 
@@ -164,28 +187,26 @@ static void sig_message_irc_action(IRC_SERVER_REC *server, const char *msg,
 	if (settings_get_bool("emphasis"))
 		msg = freemsg = expand_emphasis(item, msg);
 
-	recoded = recode_in(SERVER(server), msg, target);
 	if (ischannel(*target)) {
 		/* channel action */
-		if (window_item_is_active(item)) {
+		if (window_item_is_active(item) && target == oldtarget) {
 			/* message to active channel in window */
 			printformat(server, target, level,
-				    IRCTXT_ACTION_PUBLIC, nick, recoded);
+				    IRCTXT_ACTION_PUBLIC, nick, msg);
 		} else {
-			/* message to not existing/active channel */
+			/* message to not existing/active channel, or to @/+ */
 			printformat(server, target, level,
 				    IRCTXT_ACTION_PUBLIC_CHANNEL,
-				    nick, target, recoded);
+				    nick, oldtarget, msg);
 		}
 	} else {
 		/* private action */
 		printformat(server, nick, MSGLEVEL_ACTIONS | MSGLEVEL_MSGS,
 			    item == NULL ? IRCTXT_ACTION_PRIVATE :
 			    IRCTXT_ACTION_PRIVATE_QUERY,
-			    nick, address == NULL ? "" : address, recoded);
+			    nick, address == NULL ? "" : address, msg);
 	}
 	
-	g_free(recoded);
 	g_free_not_null(freemsg);
 }
 
@@ -194,7 +215,7 @@ static void sig_message_own_notice(IRC_SERVER_REC *server, const char *msg,
 {
         /* ugly: recode the sent message back for printing */
         char *recoded = recode_in(SERVER(server), msg, target);
-	printformat(server, skip_target(target), MSGLEVEL_NOTICES |
+	printformat(server, skip_target(server, target), MSGLEVEL_NOTICES |
 		    MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
 		    IRCTXT_OWN_NOTICE, target, recoded);
         g_free(recoded);
@@ -205,20 +226,17 @@ static void sig_message_irc_notice(SERVER_REC *server, const char *msg,
 				   const char *target)
 {
 	const char *oldtarget;
-	char *recoded;
 	
 	oldtarget = target;
-	target = skip_target(target);
+	target = skip_target(IRC_SERVER(server), target);
 
 	if (address == NULL || *address == '\0') {
 		/* notice from server */
-		recoded = recode_in(SERVER(server), msg, NULL);
 		if (!ignore_check(server, nick, "",
 				  target, msg, MSGLEVEL_SNOTES)) {
 			printformat(server, target, MSGLEVEL_SNOTES,
-				    IRCTXT_NOTICE_SERVER, nick, recoded);
+				    IRCTXT_NOTICE_SERVER, nick, msg);
 		}
-		g_free(recoded);
                 return;
 	}
 
@@ -227,25 +245,23 @@ static void sig_message_irc_notice(SERVER_REC *server, const char *msg,
 			 msg, MSGLEVEL_NOTICES))
 		return;
 
-        recoded = recode_in(SERVER(server), msg, target);
         if (ischannel(*target)) {
 		/* notice in some channel */
 		printformat(server, target, MSGLEVEL_NOTICES,
-			    IRCTXT_NOTICE_PUBLIC, nick, oldtarget, recoded);
+			    IRCTXT_NOTICE_PUBLIC, nick, oldtarget, msg);
 	} else {
 		/* private notice */
 		privmsg_get_query(SERVER(server), nick, FALSE,
 				  MSGLEVEL_NOTICES);
 		printformat(server, nick, MSGLEVEL_NOTICES,
-			    IRCTXT_NOTICE_PRIVATE, nick, address, recoded);
+			    IRCTXT_NOTICE_PRIVATE, nick, address, msg);
 	}
-	g_free(recoded);
 }
 
 static void sig_message_own_ctcp(IRC_SERVER_REC *server, const char *cmd,
 				 const char *data, const char *target)
 {
-	printformat(server, target, MSGLEVEL_CTCPS |
+	printformat(server, skip_target(server, target), MSGLEVEL_CTCPS |
 		    MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
 		    IRCTXT_OWN_CTCP, target, cmd, data);
 }
@@ -254,8 +270,12 @@ static void sig_message_irc_ctcp(IRC_SERVER_REC *server, const char *cmd,
 				 const char *data, const char *nick,
 				 const char *addr, const char *target)
 {
+	const char *oldtarget;
+
+	oldtarget = target;
+	target = skip_target(server, target);
 	printformat(server, ischannel(*target) ? target : nick, MSGLEVEL_CTCPS,
-		    IRCTXT_CTCP_REQUESTED, nick, addr, cmd, data, target);
+		    IRCTXT_CTCP_REQUESTED, nick, addr, cmd, data, oldtarget);
 }
 
 void fe_irc_messages_init(void)
