@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -105,7 +105,7 @@ FORMAT_REC fecommon_irc_formats[] = {
 	{ "whois_channels", "{whois channels %|$1}", 2, { 0, 0 } },
 	{ "whois_away", "{whois away %|$1}", 2, { 0, 0 } },
 	{ "whois_special", "{whois  %|$1}", 2, { 0, 0 } },
-	{ "whois_extra", "{whois  %|$1}", 2, { 0, 0 } },
+	{ "whois_extra", "{whois account %|$1}", 2, { 0, 0 } },
 	{ "end_of_whois", "End of WHOIS", 1, { 0 } },
 	{ "end_of_whowas", "End of WHOWAS", 1, { 0 } },
 	{ "whois_not_found", "There is no such nick $0", 1, { 0 } },
@@ -161,6 +161,7 @@ FORMAT_REC fecommon_irc_formats[] = {
 	{ "unsilenced", "Unsilenced {nick $0}", 1, { 0 } },
 	{ "silence_line", "{nick $0}: silence {ban $1}", 2, { 0, 0 } },
 	{ "ask_oper_pass", "Operator password:", 0 },
+	{ "accept_list", "Accepted users: {hilight $0}", 1, { 0 } },
 
 	{ NULL, NULL, 0 }
 };
