@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -398,8 +398,10 @@ static int check_pasting(unichar key, int diff)
 		paste_line_count = 0;
 		paste_keycount = 0;
 		g_array_set_size(paste_buffer, 0);
-		if (prev_key != '\r' && prev_key != '\n')
+		if (prev_key != '\r' && prev_key != '\n') {
+			paste_keycount++;
 			g_array_append_val(paste_buffer, prev_key);
+		}
 	} else if (paste_state > 0 && diff > paste_detect_time &&
 		   paste_line_count == 0) {
 		/* reset paste state */
@@ -415,13 +417,15 @@ static int check_pasting(unichar key, int diff)
 	}
 
 	g_array_append_val(paste_buffer, key);
-	if (key == '\r' || key == '\n') {
+	if ((key == '\r' || key == '\n') &&
+	    (prev_key != '\r' && prev_key != '\n')) {
 		if (paste_state == 1 &&
 		    paste_keycount < paste_detect_keycount) {
 			/* not enough keypresses to determine if this is
 			   pasting or not. don't reset paste keycount, but
 			   send this line as non-pasted */
 			g_array_set_size(paste_buffer, 0);
+			*paste_entry = '\0';
 			return FALSE;
 		}
 
@@ -466,6 +470,7 @@ static void sig_gui_key_pressed(gpointer keyp)
 
 	if (check_pasting(key, diff)) {
 		last_keypress = now;
+		prev_key = key;
 		return;
 	}
 
@@ -692,8 +697,7 @@ static void key_upcase_word(void)
 static void key_delete_character(void)
 {
 	if (gui_entry_get_pos(active_entry) < active_entry->text_len) {
-		gui_entry_move_pos(active_entry, 1);
-		gui_entry_erase(active_entry, 1, FALSE);
+		gui_entry_erase_cell(active_entry);
 	}
 }
 
@@ -1083,6 +1087,15 @@ void gui_readline_init(void)
 	key_bind("key", NULL, "meta2-1;5D", "cleft", (SIGNAL_FUNC) key_combo);
 	key_bind("key", NULL, "meta2-1;5C", "cright", (SIGNAL_FUNC) key_combo);
 
+	key_bind("key", NULL, "meta2-1;3A", "mup", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta2-1;3B", "mdown", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta2-1;3D", "mleft", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta2-1;3C", "mright", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta-up", "mup", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta-down", "mdown", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta-left", "mleft", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta-right", "mright", (SIGNAL_FUNC) key_combo);
+
 	/* cursor movement */
 	key_bind("backward_character", "", "left", NULL, (SIGNAL_FUNC) key_backward_character);
 	key_bind("forward_character", "", "right", NULL, (SIGNAL_FUNC) key_forward_character);
@@ -1128,10 +1141,10 @@ void gui_readline_init(void)
         /* window managing */
 	key_bind("previous_window", "Previous window", "^P", NULL, (SIGNAL_FUNC) key_previous_window);
 	key_bind("next_window", "Next window", "^N", NULL, (SIGNAL_FUNC) key_next_window);
-	key_bind("upper_window", "Upper window", "meta-up", NULL, (SIGNAL_FUNC) key_upper_window);
-	key_bind("lower_window", "Lower window", "meta-down", NULL, (SIGNAL_FUNC) key_lower_window);
-	key_bind("left_window", "Window in left", "meta-left", NULL, (SIGNAL_FUNC) key_left_window);
-	key_bind("right_window", "Window in right", "meta-right", NULL, (SIGNAL_FUNC) key_right_window);
+	key_bind("upper_window", "Upper window", "mup", NULL, (SIGNAL_FUNC) key_upper_window);
+	key_bind("lower_window", "Lower window", "mdown", NULL, (SIGNAL_FUNC) key_lower_window);
+	key_bind("left_window", "Window in left", "mleft", NULL, (SIGNAL_FUNC) key_left_window);
+	key_bind("right_window", "Window in right", "mright", NULL, (SIGNAL_FUNC) key_right_window);
 	key_bind("active_window", "Go to next window with the highest activity", "meta-a", NULL, (SIGNAL_FUNC) key_active_window);
 	key_bind("next_window_item", "Next channel/query", "^X", NULL, (SIGNAL_FUNC) key_next_window_item);
 	key_bind("previous_window_item", "Previous channel/query", NULL, NULL, (SIGNAL_FUNC) key_previous_window_item);
