@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -684,8 +684,8 @@ void statusbar_item_default_handler(SBAR_ITEM_REC *item, int get_size_only,
 				    int escape_vars)
 {
 	SERVER_REC *server;
-	WI_ITEM_REC *wiitem;
-        char *tmpstr, *tmpstr2;
+	WI_ITEM_REC *wiitem; 
+	char *tmpstr, *tmpstr2;
 	int len;
 
 	if (str == NULL)
@@ -732,7 +732,7 @@ void statusbar_item_default_handler(SBAR_ITEM_REC *item, int get_size_only,
                         /* they're forcing us smaller than minimum size.. */
 			len = format_real_length(tmpstr, item->size);
                         tmpstr[len] = '\0';
-		} else {
+		}
 			/* make sure the str is big enough to fill the
 			   requested size, so it won't corrupt screen */
 			len = format_get_length(tmpstr);
@@ -748,7 +748,6 @@ void statusbar_item_default_handler(SBAR_ITEM_REC *item, int get_size_only,
 				g_free(tmpstr);
 				tmpstr = tmpstr2;
 			}
-		}
 
 		tmpstr2 = update_statusbar_bg(tmpstr, item->bar->color);
 		gui_printtext(item->xpos, item->bar->real_ypos, tmpstr2);
