@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -180,10 +180,10 @@ static CHANNEL_REC *irc_channel_find_server(SERVER_REC *server,
                         continue;
 
 		/* check both !ABCDEchannel and !channel */
-		if (g_strcasecmp(channel, rec->name) == 0)
+		if (IRC_SERVER(server)->nick_comp_func(channel, rec->name) == 0)
 			return rec;
 
-		if (g_strcasecmp(channel, rec->visible_name) == 0)
+		if (IRC_SERVER(server)->nick_comp_func(channel, rec->visible_name) == 0)
 			return rec;
 	}
 
