@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -102,7 +102,7 @@ static void send_message(SERVER_REC *server, const char *target,
 static void server_init(IRC_SERVER_REC *server)
 {
 	IRC_SERVER_CONNECT_REC *conn;
-	char hostname[100], *address, *ptr, *username, *cmd;
+	char *address, *ptr, *username, *cmd;
 	GTimeVal now;
 
 	g_return_if_fail(server != NULL);
@@ -135,9 +135,6 @@ static void server_init(IRC_SERVER_REC *server)
 	g_free(cmd);
 
 	/* send user/realname */
-	if (gethostname(hostname, sizeof(hostname)) != 0 || *hostname == '\0')
-		strcpy(hostname, "xx");
-
 	address = server->connrec->address;
         ptr = strrchr(address, ':');
 	if (ptr != NULL) {
@@ -148,29 +145,11 @@ static void server_init(IRC_SERVER_REC *server)
 			address = "x";
 	}
 
-	/* Replace ':' with '_' in our own hostname (the same IPv6 problem) */
-	/* Replace '\n' and '\r' with '\0' in our own hostname to fix multiple PTR addreses */
-	for (ptr = hostname; *ptr != '\0'; ptr++) {
-		if (*ptr == ':') *ptr = '_';
-		if (*ptr == '\n') *ptr = '\0';
-		if (*ptr == '\r') *ptr = '\0';
-	}
-
-	/* don't allow hostname to begin with number or '+', '-'. those
-	   can be interpreted as ircnet ircd's mode parameter.
-
-	   username/hostname parameters should probably be configurable since
-	   they're only needed with some old servers which uses them to count
-	   unique users. */
-	if ((hostname[0] >= '0' && hostname[0] <= '9') ||
-	    hostname[0] == '+' || hostname[0] == '-')
-		hostname[0] = '_';
-
 	username = g_strdup(conn->username);
 	ptr = strchr(username, ' ');
 	if (ptr != NULL) *ptr = '\0';
 
-	cmd = g_strdup_printf("USER %s %s %s :%s", username, hostname, address, conn->realname);
+	cmd = g_strdup_printf("USER %s %s %s :%s", username, username, address, conn->realname);
 	irc_send_cmd_now(server, cmd);
 	g_free(cmd);
 	g_free(username);
@@ -309,7 +288,7 @@ static void sig_connected(IRC_SERVER_REC *server)
 	server->send_message = send_message;
 	server->query_find_func =
 		(QUERY_REC *(*)(SERVER_REC *, const char *)) irc_query_find;
-	server->nick_comp_func = irc_nickcmp_ascii;
+	server->nick_comp_func = irc_nickcmp_rfc1459;
 
 	server->splits = g_hash_table_new((GHashFunc) g_istr_hash,
 					  (GCompareFunc) g_istr_equal);
@@ -717,6 +696,39 @@ static void event_channels_formed(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
+static void event_hosthidden(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *newhost, *p, *newuserhost;
+
+	g_return_if_fail(server != NULL);
+
+	params = event_get_params(data, 2, NULL, &newhost);
+	/* do a sanity check */
+	if (!strchr(newhost, '*') && !strchr(newhost, '?') &&
+			!strchr(newhost, '!') && !strchr(newhost, '#') &&
+			!strchr(newhost, '&') && !strchr(newhost, ' ') &&
+			*newhost != '\0' && *newhost != '@' &&
+			*newhost != ':' && *newhost != '-' &&
+			newhost[strlen(newhost) - 1] != '-') {
+		if (strchr(newhost, '@')) {
+			newuserhost = g_strdup(newhost);
+			g_free(server->userhost);
+			server->userhost = newuserhost;
+		} else if (server->userhost != NULL) {
+			/* no user@, only process if we know the user@
+			 * already
+			 */
+			p = strchr(server->userhost, '@');
+			if (p == NULL)
+				p = server->userhost;
+			newuserhost = g_strdup_printf("%.*s@%s", (int)(p - server->userhost), server->userhost, newhost);
+			g_free(server->userhost);
+			server->userhost = newuserhost;
+		}
+	}
+	g_free(params);
+}
+
 static void event_server_banned(IRC_SERVER_REC *server, const char *data)
 {
 	g_return_if_fail(server != NULL);
@@ -729,7 +741,9 @@ static void event_error(IRC_SERVER_REC *server, const char *data)
 	g_return_if_fail(server != NULL);
 
 	if (!server->connected && (stristr(data, "Unauthorized") != NULL ||
-				   stristr(data, "K-lined") != NULL))
+				   stristr(data, "K-lined") != NULL ||
+				   stristr(data, "Banned") != NULL ||
+				   stristr(data, "Bad user info") != NULL))
 		server->banned = TRUE;
 }
 
@@ -813,6 +827,7 @@ void irc_servers_init(void)
 	signal_add_last("event 376", (SIGNAL_FUNC) event_end_of_motd);
 	signal_add_last("event 422", (SIGNAL_FUNC) event_end_of_motd); /* no motd */
 	signal_add("event 254", (SIGNAL_FUNC) event_channels_formed);
+	signal_add("event 396", (SIGNAL_FUNC) event_hosthidden);
 	signal_add("event 465", (SIGNAL_FUNC) event_server_banned);
 	signal_add("event error", (SIGNAL_FUNC) event_error);
 	signal_add("event ping", (SIGNAL_FUNC) event_ping);
@@ -838,6 +853,7 @@ void irc_servers_deinit(void)
 	signal_remove("event 376", (SIGNAL_FUNC) event_end_of_motd);
 	signal_remove("event 422", (SIGNAL_FUNC) event_end_of_motd); /* no motd */
 	signal_remove("event 254", (SIGNAL_FUNC) event_channels_formed);
+	signal_remove("event 396", (SIGNAL_FUNC) event_hosthidden);
 	signal_remove("event 465", (SIGNAL_FUNC) event_server_banned);
 	signal_remove("event error", (SIGNAL_FUNC) event_error);
 	signal_remove("event ping", (SIGNAL_FUNC) event_ping);
