@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define NEED_PERL_H
@@ -526,6 +526,16 @@ static void perl_script_fill_hash(HV *hv, PERL_SCRIPT_REC *script)
 	hv_store(hv, "data", 4, new_pv(script->data), 0);
 }
 
+static void remove_newlines(char *str)
+{
+	char *writing = str;
+
+	for (;*str;str++)
+		if (*str != '\n' && *str != '\r')
+			*(writing++) = *str;
+	*writing = '\0';
+}
+
 void perl_command(const char *cmd, SERVER_REC *server, WI_ITEM_REC *item)
 {
         const char *cmdchars;
@@ -540,6 +550,14 @@ void perl_command(const char *cmd, SERVER_REC *server, WI_ITEM_REC *item)
 		sendcmd = g_strdup_printf("%c%s", *cmdchars, cmd);
 	}
 
+	/* remove \r and \n from commands,
+	   to make it harder to introduce a security bug in a script */
+	if(strpbrk(sendcmd, "\r\n")) {
+		if (sendcmd == cmd)
+			sendcmd = strdup(cmd);
+		remove_newlines(sendcmd);
+	}
+
 	signal_emit("send command", 3, sendcmd, server, item);
 	if (sendcmd != cmd) g_free(sendcmd);
 }
@@ -566,13 +584,8 @@ static void perl_register_protocol(CHAT_PROTOCOL_REC *rec)
 	chat_type = chat_protocol_lookup(rec->name);
 	g_return_if_fail(chat_type >= 0);
 
-#if GLIB_MAJOR_VERSION < 2
-	name = g_strdup(rec->name);
-	g_strdown(name+1);
-#else
 	name = g_ascii_strdown(rec->name,-1);
 	*name = *(rec->name);
-#endif
 
 	/* window items: channel, query */
 	type = module_get_uniq_id_str("WINDOW ITEM TYPE", "CHANNEL");
